@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2004-2020 ircd-hybrid development team
+ *  Copyright (c) 2004-2024 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 /*! \file m_etrace.c
  * \brief Includes required functions for processing the ETRACE command.
- * \version $Id: m_etrace.c 9396 2020-05-14 19:26:35Z michael $
  */
 
 #include "stdinc.h"
@@ -34,9 +33,8 @@
 #include "server.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "conf.h"
-#include "patchlevel.h"
 
 
 /* report_this_status()
@@ -48,56 +46,49 @@
  * side effects - NONE
  */
 static void
-report_this_status(struct Client *source_p, const struct Client *target_p)
+trace_send_status(struct Client *source, const struct Client *target)
 {
-  if (target_p->status != STAT_CLIENT)
+  if (target->status != STAT_CLIENT)
     return;
 
-  sendto_one_numeric(source_p, &me, RPL_ETRACE,
-                     HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
-                     get_client_class(&target_p->connection->confs),
-                     target_p->name,
-                     target_p->username,
-                     target_p->host,
-                     target_p->sockhost,
-                     target_p->info);
+  sendto_one_numeric(source, &me, RPL_ETRACE,
+                     HasUMode(target, UMODE_OPER) ? "Oper" : "User",
+                     class_get_name(&target->connection->confs),
+                     target->name, target->username, target->host, target->sockhost, target->info);
 }
 
 /*
  * do_etrace()
  */
 static void
-do_etrace(struct Client *source_p, const char *name)
+do_etrace(struct Client *source, const char *name)
 {
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE, "ETRACE requested by %s (%s@%s) [%s]",
+                       source->name, source->username, source->host, source->servptr->name);
+
   bool doall = false;
-  dlink_node *node;
-
-  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
-                       "ETRACE requested by %s (%s@%s) [%s]",
-                       source_p->name, source_p->username,
-                       source_p->host, source_p->servptr->name);
-
   if (EmptyString(name))
     doall = true;
   else if (match(name, me.name) == 0)
     doall = true;
-  else if (!MyClient(source_p) && strcmp(name, me.id) == 0)
+  else if (!MyClient(source) && strcmp(name, me.id) == 0)
     doall = true;
 
-  DLINK_FOREACH(node, local_client_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, local_client_list.head)
   {
-    const struct Client *target_p = node->data;
+    const struct Client *target = node->data;
 
-    if (doall == true || match(name, target_p->name) == 0)
-      report_this_status(source_p, target_p);
+    if (doall || match(name, target->name) == 0)
+      trace_send_status(source, target);
   }
 
-  sendto_one_numeric(source_p, &me, RPL_ETRACEEND, me.name);
+  sendto_one_numeric(source, &me, RPL_ETRACEEND, me.name);
 }
 
 /*! \brief ETRACE command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -108,30 +99,30 @@ do_etrace(struct Client *source_p, const char *name)
  *      - parv[2] = nick or server name to forward the etrace to
  */
 static void
-mo_etrace(struct Client *source_p, int parc, char *parv[])
+mo_etrace(struct Client *source, int parc, char *parv[])
 {
   if (parc > 2)
-    if (server_hunt(source_p, ":%s ETRACE %s :%s", 2, parv)->ret != HUNTED_ISME)
+    if (server_hunt(source, ":%s ETRACE %s :%s", 2, parv)->ret != HUNTED_ISME)
       return;
 
-  const struct server_hunt *hunt = server_hunt(source_p, ":%s ETRACE :%s", 1, parv);
+  const struct server_hunt *hunt = server_hunt(source, ":%s ETRACE :%s", 1, parv);
   switch (hunt->ret)
   {
     case HUNTED_PASS:
-      sendto_one_numeric(source_p, &me, RPL_TRACELINK,
-                         PATCHLEVEL, hunt->target_p->name, hunt->target_p->from->name);
+      sendto_one_numeric(source, &me, RPL_TRACELINK,
+                         IRCD_VERSION, hunt->target->name, hunt->target->from->name);
       break;
     case HUNTED_ISME:
-      do_etrace(source_p, parv[1]);
+      do_etrace(source, parv[1]);
       break;
     default:
       break;
   }
 }
 
-static struct Message etrace_msgtab =
+static struct Command command_table =
 {
-  .cmd = "ETRACE",
+  .name = "ETRACE",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = mo_etrace },
@@ -140,20 +131,19 @@ static struct Message etrace_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&etrace_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&etrace_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .version = "$Revision: 9396 $",
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
