@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2020 ircd-hybrid development team
+ *  Copyright (c) 1997-2024 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,24 +21,22 @@
 
 /*! \file m_user.c
  * \brief Includes required functions for processing the USER command.
- * \version $Id: m_user.c 9585 2020-08-29 14:00:29Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "irc_string.h"
 #include "ircd.h"
-#include "numeric.h"
 #include "user.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "listener.h"
 
 
 /*! \brief USER command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -51,38 +49,38 @@
  *      - parv[4] = user's real name info
  */
 static void
-mr_user(struct Client *source_p, int parc, char *parv[])
+mr_user(struct Client *source, int parc, char *parv[])
 {
   const char *const username = parv[1];
   const char *const realname = parv[4];
 
-  assert(IsUnknown(source_p));
+  assert(IsUnknown(source));
 
-  if (listener_has_flag(source_p->connection->listener, LISTENER_SERVER) == true)
+  if (listener_has_flag(source->connection->listener, LISTENER_SERVER))
   {
-    exit_client(source_p, "Use a different port");
+    exit_client(source, "Use a different port");
     return;
   }
 
-  if (!HasFlag(source_p, FLAGS_GOTID))
+  if (!HasFlag(source, FLAGS_GOTID))
   {
     char *p = strchr(username, '@');
     if (p)
       *p = '\0';
 
-    strlcpy(source_p->username, username, sizeof(source_p->username));
+    strlcpy(source->username, username, sizeof(source->username));
   }
 
-  strlcpy(source_p->info, realname, sizeof(source_p->info));
-  source_p->connection->registration &= ~REG_NEED_USER;
+  strlcpy(source->info, realname, sizeof(source->info));
+  source->connection->registration &= ~REG_NEED_USER;
 
-  if (source_p->connection->registration == 0)
-    register_local_user(source_p);
+  if (source->connection->registration == 0)
+    user_register_local(source);
 }
 
-static struct Message user_msgtab =
+static struct Command command_table =
 {
-  .cmd = "USER",
+  .name = "USER",
   .handlers[UNREGISTERED_HANDLER] = { .handler = mr_user, .args_min = 5 },
   .handlers[CLIENT_HANDLER] = { .handler = m_registered },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -91,20 +89,19 @@ static struct Message user_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&user_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&user_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .version = "$Revision: 9585 $",
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
