@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2020 ircd-hybrid development team
+ *  Copyright (c) 1997-2024 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 /*! \file send.c
  * \brief Functions for sending messages.
- * \version $Id: send.c 9835 2020-12-25 10:12:13Z michael $
  */
 
 #include "stdinc.h"
@@ -31,15 +30,15 @@
 #include "client.h"
 #include "dbuf.h"
 #include "irc_string.h"
-#include "ircd.h"
-#include "s_bsd.h"
+#include "event.h"
+#include "comm.h"
 #include "server_capab.h"
 #include "conf_class.h"
 #include "log.h"
+#include "conf.h"
+#include "address.h"
 
-
-static uintmax_t current_serial;
-
+static uintmax_t send_marker;
 
 /* send_format()
  *
@@ -86,14 +85,14 @@ send_message(struct Client *to, struct dbuf_block *buffer)
   assert(to != &me);
   assert(MyConnect(to));
 
-  if (dbuf_length(&to->connection->buf_sendq) + buffer->size > get_sendq(&to->connection->confs))
+  if (dbuf_length(&to->connection->buf_sendq) + buffer->size > class_get_sendq(&to->connection->confs))
   {
     if (IsServer(to))
       sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
                            "Max SendQ limit exceeded for %s: %zu > %u",
                            client_get_name(to, HIDE_IP),
                            (dbuf_length(&to->connection->buf_sendq) + buffer->size),
-                           get_sendq(&to->connection->confs));
+                           class_get_sendq(&to->connection->confs));
 
     if (IsClient(to))
       AddFlag(to, FLAGS_SENDQEX);
@@ -167,8 +166,6 @@ sendq_unblocked(fde_t *F, void *data)
 void
 send_queued_write(struct Client *to)
 {
-  ssize_t retlen;
-
   /*
    * Once socket is marked dead, we cannot start writing to it,
    * even if the error is removed...
@@ -179,6 +176,7 @@ send_queued_write(struct Client *to)
   /* Next, lets try to write some data */
   while (dbuf_length(&to->connection->buf_sendq))
   {
+    ssize_t retlen;
     bool want_read = false;
     const struct dbuf_block *first = to->connection->buf_sendq.blocks.head->data;
 
@@ -187,7 +185,7 @@ send_queued_write(struct Client *to)
       retlen = tls_write(&to->connection->fd->tls, first->data + to->connection->buf_sendq.pos,
                                                    first->size - to->connection->buf_sendq.pos, &want_read);
 
-      if (want_read == true)
+      if (want_read)
         return;  /* Retry later, don't register for write events */
     }
     else
@@ -196,7 +194,7 @@ send_queued_write(struct Client *to)
 
     if (retlen <= 0)
     {
-      if (retlen < 0 && comm_ignore_errno(errno) == true)
+      if (retlen < 0 && comm_ignore_errno(errno))
       {
         AddFlag(to, FLAGS_BLOCKED);
         /* We have a non-fatal error, reschedule a write */
@@ -225,16 +223,14 @@ send_queued_write(struct Client *to)
 void
 sendto_one(struct Client *to, const char *pattern, ...)
 {
-  va_list args;
-
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
 
-  va_start(args, pattern);
-
   struct dbuf_block *buffer = dbuf_alloc();
-  send_format(buffer, pattern, args);
 
+  va_list args;
+  va_start(args, pattern);
+  send_format(buffer, pattern, args);
   va_end(args);
 
   send_message(to->from, buffer);
@@ -245,7 +241,6 @@ sendto_one(struct Client *to, const char *pattern, ...)
 void
 sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numerics numeric, ...)
 {
-  va_list args;
 
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
@@ -257,6 +252,7 @@ sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numeri
   struct dbuf_block *buffer = dbuf_alloc();
   dbuf_put_fmt(buffer, ":%s %03d %s ", ID_or_name(from, to), numeric & ~SND_EXPLICIT, dest);
 
+  va_list args;
   va_start(args, numeric);
 
   const char *numstr;
@@ -276,8 +272,6 @@ sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numeri
 void
 sendto_one_notice(struct Client *to, const struct Client *from, const char *pattern, ...)
 {
-  va_list args;
-
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
 
@@ -288,6 +282,7 @@ sendto_one_notice(struct Client *to, const struct Client *from, const char *patt
   struct dbuf_block *buffer = dbuf_alloc();
   dbuf_put_fmt(buffer, ":%s NOTICE %s ", ID_or_name(from, to), dest);
 
+  va_list args;
   va_start(args, pattern);
   send_format(buffer, pattern, args);
   va_end(args);
@@ -310,11 +305,9 @@ sendto_one_notice(struct Client *to, const struct Client *from, const char *patt
  */
 void
 sendto_channel_butone(struct Client *one, const struct Client *from,
-                      struct Channel *channel, unsigned int type,
+                      struct Channel *channel, int rank,
                       const char *pattern, ...)
 {
-  va_list args_l, args_r;
-  dlink_node *node;
   struct dbuf_block *buffer_l = dbuf_alloc();
   struct dbuf_block *buffer_r = dbuf_alloc();
 
@@ -325,6 +318,7 @@ sendto_channel_butone(struct Client *one, const struct Client *from,
 
   dbuf_put_fmt(buffer_r, ":%s ", from->id);
 
+  va_list args_l, args_r;
   va_start(args_l, pattern);
   va_start(args_r, pattern);
   send_format(buffer_l, pattern, args_l);
@@ -332,9 +326,10 @@ sendto_channel_butone(struct Client *one, const struct Client *from,
   va_end(args_l);
   va_end(args_r);
 
-  ++current_serial;
+  ++send_marker;
 
-  DLINK_FOREACH(node, channel->members.head)
+  list_node_t *node;
+  LIST_FOREACH(node, channel->members.head)
   {
     struct ChannelMember *member = node->data;
     struct Client *target = member->client;
@@ -347,7 +342,7 @@ sendto_channel_butone(struct Client *one, const struct Client *from,
     if (one && (target->from == one->from))
       continue;
 
-    if (type && (member->flags & type) == 0)
+    if (rank && member_highest_rank(member) < rank)
       continue;
 
     if (HasUMode(target, UMODE_DEAF))
@@ -355,10 +350,10 @@ sendto_channel_butone(struct Client *one, const struct Client *from,
 
     if (MyConnect(target))
       send_message(target, buffer_l);
-    else if (target->from->connection->serial != current_serial)
+    else if (target->from->connection->send_marker != send_marker)
       send_message_remote(target->from, from, buffer_r);
 
-    target->from->connection->serial = current_serial;
+    target->from->connection->send_marker = send_marker;
   }
 
   dbuf_ref_free(buffer_l);
@@ -389,15 +384,15 @@ sendto_server(const struct Client *one,
               const unsigned int nocapab,
               const char *format, ...)
 {
-  va_list args;
-  dlink_node *node;
   struct dbuf_block *buffer = dbuf_alloc();
 
+  va_list args;
   va_start(args, format);
   send_format(buffer, format, args);
   va_end(args);
 
-  DLINK_FOREACH(node, local_server_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, local_server_list.head)
   {
     struct Client *client = node->data;
 
@@ -436,22 +431,22 @@ void
 sendto_common_channels_local(struct Client *user, bool touser, unsigned int poscap,
                              unsigned int negcap, const char *pattern, ...)
 {
-  va_list args;
-  dlink_node *node, *node2;
   struct dbuf_block *buffer = dbuf_alloc();
 
+  va_list args;
   va_start(args, pattern);
   send_format(buffer, pattern, args);
   va_end(args);
 
-  ++current_serial;
+  ++send_marker;
 
-  DLINK_FOREACH(node, user->channel.head)
+  list_node_t *node, *node2;
+  LIST_FOREACH(node, user->channel.head)
   {
     struct ChannelMember *member = node->data;
     struct Channel *channel = member->channel;
 
-    DLINK_FOREACH(node2, channel->members_local.head)
+    LIST_FOREACH(node2, channel->members_local.head)
     {
       struct ChannelMember *member2 = node2->data;
       struct Client *target = member2->client;
@@ -462,7 +457,7 @@ sendto_common_channels_local(struct Client *user, bool touser, unsigned int posc
       if (target == user)
         continue;
 
-      if (target->connection->serial == current_serial)
+      if (target->connection->send_marker == send_marker)
         continue;
 
       if (poscap && HasCap(target, poscap) != poscap)
@@ -471,12 +466,12 @@ sendto_common_channels_local(struct Client *user, bool touser, unsigned int posc
       if (negcap && HasCap(target, negcap))
         continue;
 
-      target->connection->serial = current_serial;
+      target->connection->send_marker = send_marker;
       send_message(target, buffer);
     }
   }
 
-  if (touser == true && MyConnect(user) && !IsDead(user))
+  if (touser && MyConnect(user) && !IsDead(user))
     if (HasCap(user, poscap) == poscap)
       send_message(user, buffer);
 
@@ -486,24 +481,24 @@ sendto_common_channels_local(struct Client *user, bool touser, unsigned int posc
 /*! \brief Send a message to members of a channel that are locally connected to this server.
  * \param one      Client to skip; can be NULL
  * \param channel    Destination channel
- * \param status   Channel member status flags clients must have
+ * \param rank     Minimum channel member rank clients must have
  * \param poscap   Positive client capabilities flags (CAP)
  * \param negcap   Negative client capabilities flags (CAP)
  * \param pattern  Format string for command arguments
  */
 void
-sendto_channel_local(const struct Client *one, struct Channel *channel, unsigned int status,
+sendto_channel_local(const struct Client *one, struct Channel *channel, int rank,
                      unsigned int poscap, unsigned int negcap, const char *pattern, ...)
 {
-  va_list args;
-  dlink_node *node;
   struct dbuf_block *buffer = dbuf_alloc();
 
+  va_list args;
   va_start(args, pattern);
   send_format(buffer, pattern, args);
   va_end(args);
 
-  DLINK_FOREACH(node, channel->members_local.head)
+  list_node_t *node;
+  LIST_FOREACH(node, channel->members_local.head)
   {
     struct ChannelMember *member = node->data;
     struct Client *target = member->client;
@@ -514,7 +509,7 @@ sendto_channel_local(const struct Client *one, struct Channel *channel, unsigned
     if (one && (target == one->from))
       continue;
 
-    if (status && (member->flags & status) == 0)
+    if (rank && member_highest_rank(member) < rank)
       continue;
 
     if (poscap && HasCap(target, poscap) != poscap)
@@ -547,10 +542,19 @@ sendto_channel_local(const struct Client *one, struct Channel *channel, unsigned
  * side effects	- NONE
  */
 static bool
-match_it(const struct Client *one, const char *mask, unsigned int what)
+match_it(const struct Client *one, const char *mask, bool host)
 {
-  if (what == MATCH_HOST)
-    return match(mask, one->host) == 0;
+  if (host)
+  {
+    struct io_addr addr;
+    int bits = 0;
+    const int ret = parse_netmask(mask, &addr, &bits);
+
+    if (ret == HM_IPV4 || ret == HM_IPV6)
+      if (address_compare(&one->addr, &addr, false, false, bits))
+        return true;
+    return match(mask, one->realhost) == 0;
+  }
 
   return match(mask, one->servptr->name) == 0;
 }
@@ -564,16 +568,15 @@ match_it(const struct Client *one, const char *mask, unsigned int what)
  */
 void
 sendto_match_butone(const struct Client *one, const struct Client *from,
-                    const char *mask, int what, const char *pattern, ...)
+                    const char *mask, bool host, const char *pattern, ...)
 {
-  va_list args_l, args_r;
-  dlink_node *node;
   struct dbuf_block *buffer_l = dbuf_alloc();
   struct dbuf_block *buffer_r = dbuf_alloc();
 
   dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
   dbuf_put_fmt(buffer_r, ":%s ", from->id);
 
+  va_list args_l, args_r;
   va_start(args_l, pattern);
   va_start(args_r, pattern);
   send_format(buffer_l, pattern, args_l);
@@ -582,7 +585,8 @@ sendto_match_butone(const struct Client *one, const struct Client *from,
   va_end(args_r);
 
   /* Scan the local clients */
-  DLINK_FOREACH(node, local_client_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, local_client_list.head)
   {
     struct Client *client = node->data;
 
@@ -592,14 +596,14 @@ sendto_match_butone(const struct Client *one, const struct Client *from,
     if (one && (client == one->from))
       continue;
 
-    if (match_it(client, mask, what) == false)
+    if (match_it(client, mask, host) == false)
       continue;
 
     send_message(client, buffer_l);
   }
 
   /* Now scan servers */
-  DLINK_FOREACH(node, local_server_list.head)
+  LIST_FOREACH(node, local_server_list.head)
   {
     struct Client *client = node->data;
 
@@ -645,22 +649,22 @@ sendto_match_butone(const struct Client *one, const struct Client *from,
  * side effects	- data sent to servers matching with capab
  */
 void
-sendto_match_servs(const struct Client *source_p, const char *mask, unsigned int capab,
+sendto_match_servs(const struct Client *source, const char *mask, unsigned int capab,
                    const char *pattern, ...)
 {
-  va_list args;
-  dlink_node *node;
   struct dbuf_block *buffer = dbuf_alloc();
 
-  dbuf_put_fmt(buffer, ":%s ", source_p->id);
+  dbuf_put_fmt(buffer, ":%s ", source->id);
 
+  va_list args;
   va_start(args, pattern);
   send_format(buffer, pattern, args);
   va_end(args);
 
-  ++current_serial;
+  ++send_marker;
 
-  DLINK_FOREACH(node, global_server_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, global_server_list.head)
   {
     struct Client *target = node->data;
 
@@ -672,10 +676,10 @@ sendto_match_servs(const struct Client *source_p, const char *mask, unsigned int
       continue;
 
     /* ... or the source */
-    if (target->from == source_p->from)
+    if (target->from == source->from)
       continue;
 
-    if (target->from->connection->serial == current_serial)
+    if (target->from->connection->send_marker == send_marker)
       continue;
 
     if (IsCapable(target->from, capab) != capab)
@@ -684,8 +688,8 @@ sendto_match_servs(const struct Client *source_p, const char *mask, unsigned int
     if (match(mask, target->name))
       continue;
 
-    target->from->connection->serial = current_serial;
-    send_message_remote(target->from, source_p, buffer);
+    target->from->connection->send_marker = send_marker;
+    send_message_remote(target->from, source, buffer);
   }
 
   dbuf_ref_free(buffer);
@@ -705,19 +709,18 @@ sendto_anywhere(struct Client *to, const struct Client *from,
                 const char *command,
                 const char *pattern, ...)
 {
-  va_list args;
-
   if (IsDead(to->from))
     return;
 
   struct dbuf_block *buffer = dbuf_alloc();
   if (MyClient(to) && IsClient(from))
-    dbuf_put_fmt(buffer, ":%s!%s@%s %s %s ", from->name, from->username,
-                 from->host, command, to->name);
+    dbuf_put_fmt(buffer, ":%s!%s@%s %s %s ",
+                 from->name, from->username, from->host, command, to->name);
   else
-    dbuf_put_fmt(buffer, ":%s %s %s ", ID_or_name(from, to),
-                 command, ID_or_name(to, to));
+    dbuf_put_fmt(buffer, ":%s %s %s ",
+                 ID_or_name(from, to), command, ID_or_name(to, to));
 
+  va_list args;
   va_start(args, pattern);
   send_format(buffer, pattern, args);
   va_end(args);
@@ -741,8 +744,6 @@ sendto_anywhere(struct Client *to, const struct Client *from,
 void
 sendto_realops_flags(unsigned int flags, int level, int type, const char *pattern, ...)
 {
-  va_list args;
-  dlink_node *node;
   const char *ntype = "???";
 
   switch (type)
@@ -761,13 +762,15 @@ sendto_realops_flags(unsigned int flags, int level, int type, const char *patter
   }
 
   struct dbuf_block *buffer = dbuf_alloc();
-  dbuf_put_fmt(buffer, ":%s NOTICE * :*** %s -- ", me.name, ntype); 
+  dbuf_put_fmt(buffer, ":%s NOTICE * :*** %s -- ", me.name, ntype);
 
+  va_list args;
   va_start(args, pattern);
   send_format(buffer, pattern, args);
   va_end(args);
 
-  DLINK_FOREACH(node, oper_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, oper_list.head)
   {
     struct Client *client = node->data;
     assert(HasUMode(client, UMODE_OPER));
@@ -802,7 +805,6 @@ sendto_realops_flags(unsigned int flags, int level, int type, const char *patter
 void
 sendto_realops_flags_ratelimited(uintmax_t *rate, const char *pattern, ...)
 {
-  va_list args;
   char buffer[IRCD_BUFSIZE];
 
   if ((event_base->time.sec_monotonic - *rate) < 20)
@@ -810,12 +812,13 @@ sendto_realops_flags_ratelimited(uintmax_t *rate, const char *pattern, ...)
 
   *rate = event_base->time.sec_monotonic;
 
+  va_list args;
   va_start(args, pattern);
   vsnprintf(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
   sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s", buffer);
-  ilog(LOG_TYPE_IRCD, "%s", buffer);
+  log_write(LOG_TYPE_IRCD, "%s", buffer);
 }
 
 /* sendto_wallops_flags()
@@ -827,23 +830,23 @@ sendto_realops_flags_ratelimited(uintmax_t *rate, const char *pattern, ...)
  * side effects - Send a wallops to local opers
  */
 void
-sendto_wallops_flags(unsigned int flags, const struct Client *source_p,
+sendto_wallops_flags(unsigned int flags, const struct Client *source,
                      const char *pattern, ...)
 {
-  va_list args;
-  dlink_node *node;
   struct dbuf_block *buffer = dbuf_alloc();
 
-  if (IsClient(source_p))
-    dbuf_put_fmt(buffer, ":%s!%s@%s WALLOPS :", source_p->name, source_p->username, source_p->host);
+  if (IsClient(source))
+    dbuf_put_fmt(buffer, ":%s!%s@%s WALLOPS :", source->name, source->username, source->host);
   else
-    dbuf_put_fmt(buffer, ":%s WALLOPS :", source_p->name);
+    dbuf_put_fmt(buffer, ":%s WALLOPS :", source->name);
 
+  va_list args;
   va_start(args, pattern);
   send_format(buffer, pattern, args);
   va_end(args);
 
-  DLINK_FOREACH(node, oper_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, oper_list.head)
   {
     struct Client *client = node->data;
     assert(client->umodes & UMODE_OPER);
