@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2020 ircd-hybrid development team
+ *  Copyright (c) 1997-2024 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 /*! \file server.c
  * \brief Server related functions.
- * \version $Id: server.c 9683 2020-10-28 18:56:24Z michael $
  */
 
 #include "stdinc.h"
@@ -32,7 +31,7 @@
 #include "irc_string.h"
 #include "ircd.h"
 #include "ircd_defs.h"
-#include "s_bsd.h"
+#include "comm.h"
 #include "packet.h"
 #include "conf.h"
 #include "server.h"
@@ -43,100 +42,8 @@
 #include "parse.h"
 
 
-dlink_list flatten_links;
 static void server_connect_callback(fde_t *, int, void *);
 
-
-/*
- * write_links_file
- *
- * inputs	- void pointer which is not used
- * output	- NONE
- * side effects	- called from an event, write out list of linked servers
- *		  but in no particular order.
- */
-void
-write_links_file(void *unused)
-{
-  char buf[IRCD_BUFSIZE];
-  dlink_node *node, *node_next;
-
-  if (EmptyString(ConfigServerHide.flatten_links_file))
-    return;
-
-  FILE *file = fopen(ConfigServerHide.flatten_links_file, "w");
-  if (file == NULL)
-  {
-    ilog(LOG_TYPE_IRCD, "Couldn't open \"%s\": %s", ConfigServerHide.flatten_links_file,
-         strerror(errno));
-    return;
-  }
-
-  DLINK_FOREACH_SAFE(node, node_next, flatten_links.head)
-  {
-    dlinkDelete(node, &flatten_links);
-    xfree(node->data);
-    free_dlink_node(node);
-  }
-
-  DLINK_FOREACH(node, global_server_list.head)
-  {
-    const struct Client *client = node->data;
-
-    /*
-     * Skip hidden servers, aswell as ourselves, since we already send
-     * ourselves in /links
-     */
-    if (IsHidden(client) || IsMe(client))
-      continue;
-
-    if (HasFlag(client, FLAGS_SERVICE) && ConfigServerHide.hide_services)
-      continue;
-
-    /*
-     * Attempt to format the file in such a way it follows the usual links output
-     * ie  "servername uplink :hops info"
-     * Mostly for aesthetic reasons - makes it look pretty in mIRC ;)
-     * - madmax
-     */
-    snprintf(buf, sizeof(buf), "%s %s :1 %s", client->name, me.name, client->info);
-    dlinkAddTail(xstrdup(buf), make_dlink_node(), &flatten_links);
-
-    strlcat(buf, "\n", sizeof(buf));
-    fputs(buf, file);
-  }
-
-  fclose(file);
-}
-
-void
-read_links_file(void)
-{
-  char buf[IRCD_BUFSIZE];
-
-  if (EmptyString(ConfigServerHide.flatten_links_file))
-    return;
-
-  FILE *file = fopen(ConfigServerHide.flatten_links_file, "r");
-  if (file == NULL)
-  {
-    ilog(LOG_TYPE_IRCD, "Couldn't open \"%s\": %s", ConfigServerHide.flatten_links_file,
-         strerror(errno));
-    return;
-  }
-
-  while (fgets(buf, sizeof(buf), file))
-  {
-    char *p = strchr(buf, '\n');
-    if (p)
-      *p = '\0';
-
-    dlinkAddTail(xstrdup(buf), make_dlink_node(), &flatten_links);
-  }
-
-  fclose(file);
-}
-
 /* server_hunt()
  *      Do the basic thing in delivering the message (command)
  *      across the relays to the specific server (server) for
@@ -157,88 +64,88 @@ read_links_file(void)
  *      returns: (see #defines)
  */
 const struct server_hunt *
-server_hunt(struct Client *source_p, const char *command, const int server, char *parv[])
+server_hunt(struct Client *client, const char *command, const int server, char *parv[])
 {
   static struct server_hunt hunt;
   struct server_hunt *const h = &hunt;
-  dlink_node *node;
   const char *const mask = parv[server];
 
   /* Assume it's me, if no server */
   if (EmptyString(mask))
   {
-    h->target_p = &me;
+    h->target = &me;
     h->ret = HUNTED_ISME;
     return h;
   }
 
-  h->target_p = find_person(source_p, mask);
-  if (h->target_p == NULL)
-    h->target_p = hash_find_server(mask);
+  h->target = find_person(client, mask);
+  if (h->target == NULL)
+    h->target = hash_find_server(mask);
 
   /*
    * These are to pickup matches that would cause the following
    * message to go in the wrong direction while doing quick fast
    * non-matching lookups.
    */
-  if (h->target_p)
-    if (h->target_p->from == source_p->from && !MyConnect(h->target_p))
-      h->target_p = NULL;
+  if (h->target)
+    if (h->target->from == client->from && !MyConnect(h->target))
+      h->target = NULL;
 
-  if (h->target_p == NULL && has_wildcards(mask))
+  if (h->target == NULL && has_wildcards(mask))
   {
-    DLINK_FOREACH(node, global_server_list.head)
+    list_node_t *node;
+    LIST_FOREACH(node, global_server_list.head)
     {
       struct Client *tmp = node->data;
 
       assert(IsMe(tmp) || IsServer(tmp));
       if (match(mask, tmp->name) == 0)
       {
-        if (tmp->from == source_p->from && !MyConnect(tmp))
+        if (tmp->from == client->from && !MyConnect(tmp))
           continue;
 
-        h->target_p = tmp;
+        h->target = tmp;
         break;
       }
     }
 
-    if (h->target_p == NULL)
+    if (h->target == NULL)
     {
-      DLINK_FOREACH(node, global_client_list.head)
+      LIST_FOREACH(node, global_client_list.head)
       {
         struct Client *tmp = node->data;
 
         assert(IsClient(tmp));
         if (match(mask, tmp->name) == 0)
         {
-          if (tmp->from == source_p->from && !MyConnect(tmp))
+          if (tmp->from == client->from && !MyConnect(tmp))
             continue;
 
-          h->target_p = tmp;
+          h->target = tmp;
           break;
         }
       }
     }
   }
 
-  if (h->target_p)
+  if (h->target)
   {
-    assert(IsMe(h->target_p) || IsServer(h->target_p) || IsClient(h->target_p));
-    if (IsMe(h->target_p) || MyClient(h->target_p))
+    assert(IsMe(h->target) || IsServer(h->target) || IsClient(h->target));
+    if (IsMe(h->target) || MyClient(h->target))
     {
       h->ret = HUNTED_ISME;
       return h;
     }
 
-    parv[server] = h->target_p->id;
-    sendto_one(h->target_p, command, source_p->id,
+    parv[server] = h->target->id;
+    sendto_one(h->target, command, client->id,
                parv[1], parv[2], parv[3], parv[4],
                parv[5], parv[6], parv[7], parv[8]);
     h->ret = HUNTED_PASS;
     return h;
   }
 
-  sendto_one_numeric(source_p, &me, ERR_NOSUCHSERVER, mask);
+  sendto_one_numeric(client, &me, ERR_NOSUCHSERVER, mask);
   h->ret = HUNTED_NOSUCH;
   return h;
 }
@@ -256,12 +163,11 @@ server_hunt(struct Client *source_p, const char *command, const int server, char
 void
 try_connections(void *unused)
 {
-  dlink_node *node;
-
-  if (GlobalSetOptions.autoconn == false)
+  if (GlobalSetOptions.autoconnect == false)
     return;
 
-  DLINK_FOREACH(node, connect_items.head)
+  list_node_t *node;
+  LIST_FOREACH(node, connect_items.head)
   {
     struct MaskItem *conf = node->data;
 
@@ -296,8 +202,8 @@ try_connections(void *unused)
       /* Move this entry to the end of the list, if not already last */
       if (node->next)
       {
-        dlinkDelete(node, &connect_items);
-        dlinkAddTail(conf, &conf->node, &connect_items);
+        list_remove(node, &connect_items);
+        list_add_tail(conf, &conf->node, &connect_items);
       }
 
       if (find_servconn_in_progress(conf->name))
@@ -357,7 +263,7 @@ struct Server *
 server_make(struct Client *client)
 {
   if (client->serv == NULL)
-    client->serv = xcalloc(sizeof(*client->serv));
+    client->serv = io_calloc(sizeof(*client->serv));
 
   return client->serv;
 }
@@ -381,15 +287,12 @@ server_make(struct Client *client)
 bool
 server_connect(struct MaskItem *conf, struct Client *by)
 {
-  char buf[HOSTIPLEN + 1];
-
-  /* Make sure conf is useful */
   assert(conf);
   assert(conf->type == CONF_SERVER);
   assert(hash_find_server(conf->name) == NULL);  /* This should have been checked by the caller */
 
   /* Still processing a DNS lookup? -> exit */
-  if (conf->dns_pending == true)
+  if (conf->dns_pending)
   {
     sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
                          "Error connecting to %s: DNS lookup for connect{} in progress.",
@@ -397,7 +300,7 @@ server_connect(struct MaskItem *conf, struct Client *by)
     return false;
   }
 
-  if (conf->dns_failed == true)
+  if (conf->dns_failed)
   {
     sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
                          "Error connecting to %s: DNS lookup for connect{} failed.",
@@ -405,16 +308,18 @@ server_connect(struct MaskItem *conf, struct Client *by)
     return false;
   }
 
+  char buf[HOSTIPLEN + 1];
   getnameinfo((const struct sockaddr *)conf->addr, conf->addr->ss_len,
               buf, sizeof(buf), NULL, 0, NI_NUMERICHOST);
-  ilog(LOG_TYPE_IRCD, "Connect to %s[%s] @%s", conf->name, conf->host, buf);
+  log_write(LOG_TYPE_IRCD, "Connect to %s[%s] @%s", conf->name, conf->host, buf);
 
   /* Create a socket for the server connection */
   int fd = comm_socket(conf->addr->ss.ss_family, SOCK_STREAM, 0);
   if (fd == -1)
   {
     /* Eek, failure to create the socket */
-    report_error(L_ALL, "opening stream socket to %s: %s", conf->name, errno);
+    log_write(LOG_TYPE_IRCD, "opening stream socket to %s: %s",
+              conf->name, strerror(errno));
     return false;
   }
 
@@ -428,7 +333,7 @@ server_connect(struct MaskItem *conf, struct Client *by)
   /* We already converted the ip once, so lets use it - stu */
   strlcpy(client->sockhost, buf, sizeof(client->sockhost));
 
-  client->ip = *conf->addr;
+  client->addr = *conf->addr;
   client->connection->fd = fd_open(fd, true, NULL);
 
   /* Server names are always guaranteed under HOSTLEN chars */
@@ -471,9 +376,11 @@ server_finish_tls_handshake(struct Client *client)
   if (conf == NULL)
   {
     sendto_realops_flags(UMODE_SERVNOTICE, L_ADMIN, SEND_NOTICE,
-                         "Lost connect{} block for %s", client_get_name(client, SHOW_IP));
+                         "Lost connect{} block for %s",
+                         client_get_name(client, SHOW_IP));
     sendto_realops_flags(UMODE_SERVNOTICE, L_OPER, SEND_NOTICE,
-                         "Lost connect{} block for %s", client_get_name(client, MASK_IP));
+                         "Lost connect{} block for %s",
+                         client_get_name(client, MASK_IP));
 
     exit_client(client, "Lost connect{} block");
     return;
@@ -482,16 +389,15 @@ server_finish_tls_handshake(struct Client *client)
   /* Next, send the initial handshake */
   SetHandshake(client);
 
-  sendto_one(client, "PASS %s TS %u %s", conf->spasswd, TS_CURRENT, me.id);
+  sendto_one(client, "PASS %s", conf->spasswd);
 
-  sendto_one(client, "CAPAB :%s", capab_get(NULL));
+  sendto_one(client, "CAPAB :%s", capab_get(NULL, true));
 
-  sendto_one(client, "SERVER %s 1 :%s%s",
-             me.name, ConfigServerHide.hidden ? "(H) " : "",
-             me.info);
+  sendto_one(client, "SERVER %s 1 %s +%s :%s",
+             me.name, me.id, ConfigServerHide.hidden ? "h" : "", me.info);
 
   /* If we get here, we're ok, so lets start reading some data */
-  read_packet(client->connection->fd, client); 
+  read_packet(client->connection->fd, client);
 }
 
 static void
@@ -536,10 +442,11 @@ server_tls_handshake(fde_t *F, void *data)
   }
 
   comm_settimeout(F, 0, NULL, NULL);
+  comm_setselect(F, COMM_SELECT_WRITE | COMM_SELECT_READ, NULL, NULL, 0);
 
   if (tls_verify_certificate(&F->tls, ConfigServerInfo.message_digest_algorithm, &client->tls_certfp) == false)
-    ilog(LOG_TYPE_IRCD, "Server %s gave bad TLS client certificate",
-         client_get_name(client, MASK_IP));
+    log_write(LOG_TYPE_IRCD, "Server %s gave bad TLS client certificate",
+              client_get_name(client, MASK_IP));
 
   server_finish_tls_handshake(client);
 }
@@ -627,12 +534,12 @@ server_connect_callback(fde_t *F, int status, void *data)
   /* Next, send the initial handshake */
   SetHandshake(client);
 
-  sendto_one(client, "PASS %s TS %u %s", conf->spasswd, TS_CURRENT, me.id);
+  sendto_one(client, "PASS %s", conf->spasswd);
 
-  sendto_one(client, "CAPAB :%s", capab_get(NULL));
+  sendto_one(client, "CAPAB :%s", capab_get(NULL, true));
 
-  sendto_one(client, "SERVER %s 1 :%s%s", me.name,
-             ConfigServerHide.hidden ? "(H) " : "", me.info);
+  sendto_one(client, "SERVER %s 1 %s +%s :%s",
+             me.name, me.id, ConfigServerHide.hidden ? "h" : "", me.info);
 
   /* If we get here, we're ok, so lets start reading some data */
   read_packet(client->connection->fd, client);
@@ -641,9 +548,9 @@ server_connect_callback(fde_t *F, int status, void *data)
 struct Client *
 find_servconn_in_progress(const char *name)
 {
-  dlink_node *ptr;
+  list_node_t *ptr;
 
-  DLINK_FOREACH(ptr, unknown_list.head)
+  LIST_FOREACH(ptr, unknown_list.head)
   {
     struct Client *cptr = ptr->data;
 
